from typing import Optional, Any

from pyckish.exceptions.validation_error import ValidationError
from pyckish.lambda_input_element import LambdaInputElement, EMPTY


class QueryParameter(LambdaInputElement):
    """
    Use this class on your Lambda Handler Function.
    Extracts a single HTTP Query String Parameter.
    """

    def __init__(self, alias: Optional[str] = None, default: Any = EMPTY()) -> None:
        super().__init__(alias=alias, default=default)

    def extract(self, event: dict, context: dict) -> Any:
        key = self.alias if self.alias is not None else self.parameter_name
        try:
            return event['queryStringParameters'][key]
        except (KeyError, AttributeError):
            if type(self.default) == EMPTY:
                raise ValidationError(f'The query string parameter "{key}" could not be found in the event')
            return self.default



from typing import Any

from pyckish import LambdaInputElement


class Headers(LambdaInputElement):
    """
    Use this class on your Lambda Handler Function.
    Extracts all HTTP Headers
    """

    def __init__(self) -> None:
        super().__init__()

    def extract(self, event: dict, context: dict) -> Any:
        try:
            argument = event['headers']
        except KeyError:
            argument = {}
        return argument



import json
from typing import Optional, Union, Any

import pydantic

from pyckish.http_elements import HTTPResponse

DictOrSetIntStr = Union[
    set[Union[int, str]],
    dict[Union[int, str], Any]
]


class ResponseHandler:

    def __init__(
            self,
            is_http: bool,
            status_code: Optional[int],
            exclude_unset: bool,
            exclude_defaults: bool,
            exclude_none: bool,
            by_alias: bool,
            include: Optional[DictOrSetIntStr],
            exclude: Optional[DictOrSetIntStr]
    ) -> None:
        self.__is_http = is_http
        self.__status_code = status_code
        self.__response_config = {
            'exclude_unset': exclude_unset,
            'exclude_defaults': exclude_defaults,
            'exclude_none': exclude_none,
            'by_alias': by_alias,
            'include': include,
            'exclude': exclude
        }

    def prepare_response(self, result: Any) -> str:
        if isinstance(result, HTTPResponse):
            result.status_code = self.__status_code if result.status_code is None else result.status_code
            return result()
        if isinstance(result, pydantic.BaseModel):
            result = json.loads(result.json(**self.__response_config))
        if self.__is_http:
            return HTTPResponse(
                body=result,
                status_code=self.__status_code
            )()
        return json.dumps(result)
